include <cnc.scad>;

box_start = [-10, -5 - cnc_excess_yminus, 0];
box_size = [125, 95, 80];
box_end = box_start + box_size;
//box_end = [cnc_base_x + 10, cnc_base_y + 15, 80];

function box_coords(x, y, z) = box_start + [box_size[0]*x, box_size[1]*y, box_size[2]*z];

bar_width = 3;
module bar(length)
{
	color([0.6, 0.6, 0.65]) difference() {
		translate([0.01, 0.01, 0.01]) cube([bar_width-0.02, bar_width-0.02, length-0.02]);
		translate([0.2, 0.2, -0.2])
			cube([bar_width-0.4, bar_width-0.4, length+0.4]);
	}
	echo("using a bar with length", length);
}

module bar_4545(length)
{
	color([0.7, 0.7, 0.75]) render() difference() {
		cube([bar_width, bar_width, length]);
		translate([0.2, 0.2, -0.2])
			cube([bar_width-0.4, bar_width-0.4, length+0.4]);
		rotate([0, 45, 0]) translate([0, -bar_width/2, -bar_width/3]) cube(2*bar_width);
		translate([0, 0, length]) mirror([0, 0, 1]) rotate([0, 45, 0]) translate([0, -bar_width/2, -bar_width/3]) cube(2*bar_width);
	}
	echo("using 45/45 bar with outer length", length);
}

module box_bare()
{
	r = 1;
	box_size = box_end - box_start;

	translate(box_start)
	{
		cylinder(r=r, h=box_size[2]);
		rotate([0, 90, 0]) cylinder(r=r, h=box_size[0]);
		rotate([-90, 0, 0]) cylinder(r=r, h=box_size[1]);
	}

	translate(box_end)
	{
		mirror([0, 0, 1]) cylinder(r=r, h=box_size[2]);
		rotate([0, -90, 0]) cylinder(r=r, h=box_size[0]);
		rotate([90, 0, 0]) cylinder(r=r, h=box_size[1]);
	}

	translate([box_start[0], box_end[1], box_end[2]])
	{
		mirror([0, 0, 1]) cylinder(r=r, h=box_size[2]);
		rotate([90, 0, 0]) cylinder(r=r, h=box_size[1]);
	}

	translate([box_end[0], box_start[1], box_start[2]])
	{
		cylinder(r=r, h=box_size[2]);
		rotate([-90, 0, 0]) cylinder(r=r, h=box_size[1]);
	}
}

// Schwingtor in XZ-Ebene, das nach +Z aufgeht und sich in die XY-Ebene legt
//
// @param h Höhe des Schwingtors (Z)
// @param b Breite des Schwingtors (X)
// @param d Dicke des Schwingtors (nur für Darstellung; Y)
// @param t Streckenteil der Tür, an der das untere Lager befestigt ist; 0 = an Unterkante, 1 = reine Klapptür (Singularität)
// @param o Öffnungsgrad, 0 = geschlossen, 1 = offen
module schwingtor(h, b, d, t, o)
{
	module door_itself()
		translate([0, -d/2, 0]) cube([b, d, h]);

	module door_on_knob()
		translate([0, 0, -t*h]) door_itself();

	hypot = (1-t)*h;
	shortened_kath = hypot*(1-o);
	angle = acos(shortened_kath / hypot);

	module opened_door()
		translate([0, 0, t*h + o*h*(1-t)]) rotate([-angle, 0, 0]) door_on_knob();

	opened_door();
}

// Viergelenkstür in Anordnung wie Schwingtor
//
// Die Bewegung dieser Tür kann durch die Koppelkurve beschrieben werden; die
// Kurve selbst wird hier aber nicht bestimmt.
//
// @param h Höhe des Schwingtors (Z)
// @param b Breite des Schwingtors (X)
// @param d Dicke des Schwingtors (nur für Darstellung; Y)
// @param t1 Position des 1. Gelenksteils auf der Tür (0 = ganz unten, 1 = ganz oben)
// @param t2 Position des 2. Gelenksteils auf der Tür
// @param p1 Position des 1. festen Gelenks in der Ebene (1. Koordinate = Abstand zur Tür in Y-Richtung, 2. Koordinate = Abstand von der Unterkante in Z-Richtung)
// @param p2 Position des 2. festen Gelenks in der Ebene
// @param alpha Öffnungswinkel am 1. Gelenk relativ zu Ausgangsstellung
module viergelenkstuer(h, b, d, t1, t2, p1, p2, alpha)
{
	module door_itself()
		translate([0, -d/2, 0]) cube([b, d, h]);

	module door_on_knob()
		translate([0, 0, -t1*h]) door_itself();

	// koordinaten der tuergelenke in ausgangssellung, im koordinatenraum von p1/p2
	t1absstart = [0, h*t1];
	t2absstart = [0, h*t2];

	l1 = sqrt(pow(t1absstart[0] - p1[0], 2) + pow(t1absstart[1] - p1[1], 2));
	l2 = sqrt(pow(t2absstart[0] - p2[0], 2) + pow(t2absstart[1] - p2[1], 2));

	alphabase = atan2((t1absstart - p1)[1], (t1absstart - p1)[0]);
	alphacomplete = alphabase - alpha; // the YZ coordinate space is left-handed, angles work the other way round here

	new_t1 = [p1[0] + cos(alphacomplete) * l1, p1[1] + sin(alphacomplete) * l1];

//	function triangle(point_a, point_b, a, b) = 
//	{
//		direct_way = point_b - point_a;
//
//		angle_at_a = 5; /* nach cosinussatz */
//		angle_base = atan2(direct_way[1], direct_way[0]);
//
//		return [point_a[0] + cos(angle_base) * a, point_a[1] + sin(angle_base) * a];
//	}
//
//	new_t2 = triangle(new_t1, p2, h*(t2 - t1), l2);

	// hardcoding this as i can't have multiline functions

	longside = p2 - new_t1;
	longside_length = sqrt(pow(longside[0], 2) + pow(longside[1], 2));
	door_innerheight = h*(t2 - t1);
	// cosinussatz: l2^2 = door_innerheight^2 + longside_length^2 - 2*door_innerheight*longside_length*cos(gamma)
	// l2^2 - door_innerheight^2 - longside_length^2 = 2*door_innerheight*longside_length*cos(gamma)
	// gamma = acos((_left_) / (2*door_innerheight*longside_length))
	echo("longside", longside_length);
	echo("door_innerheight", door_innerheight);
	echo("l2", l2);
	opening_angle = acos(-((pow(l2, 2) - pow(door_innerheight, 2) - pow(longside_length, 2)) / (2*door_innerheight*longside_length)));
	echo("opening angle", opening_angle);
	new_angle = atan2(longside[1], longside[0]) + opening_angle;

	new_t2 = [new_t1[0] + cos(new_angle) * door_innerheight, new_t1[1] + sin(new_angle) * door_innerheight];

//	% translate([0, p1[0], p1[1]]) sphere(l1);
//	% translate([0, p2[0], p2[1]]) sphere(l2);
//
//	translate([0, t1absstart[0], t1absstart[1]]) sphere(3);
//	translate([0, t2absstart[0], t2absstart[1]]) sphere(3);

	color([1, 0, 0]) translate([0, p1[0], p1[1]]) sphere(3);
	color([0, 1, 0]) translate([0, p2[0], p2[1]]) sphere(3);

	color([1, 0, 0]) translate([0, new_t1[0], new_t1[1]]) sphere(3);
	color([0, 1, 0]) translate([0, new_t2[0], new_t2[1]]) sphere(3);

	color([0, 0, 1]) translate([0, new_t1[0], new_t1[1]]) rotate([new_angle-90, 0, 0]) door_on_knob();
}

module tischplatte()
{
	x = 160;
	y = 120;
	z = 5;

	translate([-25, -25, -z])
	cube([x, y, z]);
}

cnc();
* tischplatte();
box_bare();

//door = "schwingtor";
//door = "viergelenk";
//animate = 0;
animate = 1;


parameter = 0.4;

if (door == "schwingtor")
	if(animate)
		translate(box_start) schwingtor((box_end - box_start)[2], (box_end - box_start)[0], 1, parameter, abs(2*$t-1));
	else
		for(i=[0:0.08:1])
			translate(box_start) schwingtor((box_end - box_start)[2], (box_end - box_start)[0], 1, parameter, i);

green_factor = 0.62;
red_factor = 0.32;
green_point = [15, 50];
red_point = [25, 50];

if (door == "viergelenk")
	if (animate)
		translate(box_start) viergelenkstuer((box_end - box_start)[2], (box_end - box_start)[0], 3, red_factor, green_factor, red_point, green_point, abs($t*140-70));
	else
		for(i=[1:10:80])
			translate(box_start) viergelenkstuer((box_end - box_start)[2], (box_end - box_start)[0], 1, red_factor, green_factor, red_point, green_point, i);
