include <cnc.scad>
include <kaefig.scad>

table_height = 90;

drawer_clearance = 10;
drawer_distance = bar_width*3 + drawer_clearance;

diagonal_length_side = 18/sqrt(2); // used to be box_size[1]/8
diagonal_length_back = 60/sqrt(2);

module left()
{
	// front leg
	translate(box_coords(0, 0, 0) + [-bar_width, -bar_width, 0]) {
		mirror([0, 0, 1]) bar(table_height);
	}

	// back leg
	translate(box_coords(0, 1, 0) + [-bar_width, bar_width, 0]) {
		rotate([180, 0, 0]) bar(table_height);
	}
	// surface bar
	translate(box_coords(0, 0, 0)) {
		mirror([1, 0, 0]) rotate([-90, 0, 0]) translate([0, 0, -bar_width]) bar(box_size[1] + 2*bar_width);
	}

	// diagonal bars
	translate(box_coords(0, 0, 0)) mirror([1, 0, 0]) translate([0, 0, -bar_width-diagonal_length_side]) rotate([-45, 0, 0]) rotate([0, 0, -90]) bar_4545(diagonal_length_side*sqrt(2));
	translate(box_coords(0, 1, 0)) mirror([1, 0, 0]) mirror([0, 1, 0]) translate([0, 0, -bar_width-diagonal_length_side]) rotate([-45, 0, 0]) rotate([0, 0, -90]) bar_4545(diagonal_length_side*sqrt(2));

	// drawer side cover support
	translate(box_coords(0, 0, 0) + [0, 0, -drawer_distance])
		mirror([1, 0, 0]) rotate([-90, 0, 0]) bar(box_size[1]);
}

module right()
{
	translate(box_coords(1, 0, 0)) mirror([1, 0, 0]) translate(-box_coords(0, 0, 0)) left();
}

module cnc_grid()
{
	// bars left and right of the cnc
	translate(box_coords(0.5, 0, 0)) {
		translate([cnc_base_x/2 - cnc_feet_offsetx, 0, 0]) translate([-bar_width/2, 0, 0]) rotate([-90, 0, 0]) bar(box_size[1]);
		translate([-cnc_base_x/2 + cnc_feet_offsetx, 0, 0]) translate([-bar_width/2, 0, 0]) rotate([-90, 0, 0]) bar(box_size[1]);
	}

	// bars in front of and behind the cnc
	translate(box_coords(0, 0, 0)) {
		translate([0, -bar_width/2 + 20, -bar_width]) rotate([0, 90, 0]) translate([0, 0, -bar_width]) bar(box_size[0] + 2*bar_width);
		translate([0, -bar_width/2 + box_size[1] - 20, -bar_width]) rotate([0, 90, 0]) translate([0, 0, -bar_width]) bar(box_size[0] + 2*bar_width);
	}
}

module drawer_grid()
{
	translate(box_coords(0.5, 0, 0) + [0, 0, -drawer_distance + bar_width]) {
		translate([30, 0, 0]) translate([-bar_width/2, -bar_width, 0]) rotate([-90, 0, 0]) bar(box_size[1] + 2*bar_width);
		translate([-30, 0, 0]) translate([-bar_width/2, -bar_width, 0]) rotate([-90, 0, 0]) bar(box_size[1] + 2*bar_width);
	}
}

module tisch()
{
	left();

	right();

	cnc_grid();

	drawer_grid();

	// front
	translate(box_coords(0, 0, 0)) 
		mirror([0, 1, 0]) rotate([0, 90, 0]) translate([0, 0, -bar_width]) bar(box_size[0] + 2*bar_width);
	translate(box_coords(0, 0, 0) + [0, 0, -drawer_distance]) 
		mirror([0, 1, 0]) rotate([0, 90, 0]) bar(box_size[0]);

	// back
	translate(box_coords(0, 1, 0))
		rotate([0, 90, 0]) translate([0, 0, -bar_width]) bar(box_size[0] + 2*bar_width);
	translate(box_coords(0, 1, 0) + [0, 0, -drawer_distance])
		rotate([0, 90, 0]) bar(box_size[0]);

	// diagonal bars back
	translate(box_coords(0, 1, 0) + [0, 0, -drawer_distance]) translate([diagonal_length_back, 0, -bar_width]) rotate([0, 225, 0]) bar_4545(diagonal_length_back*sqrt(2));
	translate(box_coords(1, 1, 0) + [0, 0, -drawer_distance]) mirror([1, 0, 0]) translate([diagonal_length_back, 0, -bar_width]) rotate([0, 225, 0]) bar_4545(diagonal_length_back*sqrt(2));


//	// diagonal bars front
//	translate(box_coords(0, 0, 0) + [0, 0, -drawer_distance]) mirror([0, 1, 0]) translate([diagonal_length_side, 0, -bar_width]) rotate([0, 225, 0]) bar_4545(diagonal_length_side*sqrt(2));
//	translate(box_coords(1, 0, 0) + [0, 0, -drawer_distance]) mirror([0, 1, 0]) mirror([1, 0, 0]) translate([diagonal_length_side, 0, -bar_width]) rotate([0, 225, 0]) bar_4545(diagonal_length_side*sqrt(2));
}

tisch();
